@@ -35,6 +35,10 @@ static gboolean gail_deselect_watcher   (GSignalInvocationHint *ihint,
                                          guint                  n_param_values,
                                          const GValue          *param_values,
                                          gpointer               data);
+static gboolean gail_switch_page_watcher(GSignalInvocationHint *ihint,
+                                         guint                  n_param_values,
+                                         const GValue          *param_values,
+                                         gpointer               data);
 static AtkObject* gail_get_accessible_for_widget (GtkWidget    *widget,
                                                   gboolean     *transient);
 static void     gail_finish_select       (GtkWidget            *widget);
@@ -46,6 +50,7 @@ static void     gail_focus_notify_when_idle (GtkWidget            *widget);
 static void     gail_focus_tracker_init ();
 
 static GtkWidget* focus_widget = NULL;
+static GtkWidget* next_focus_widget = NULL;
 static GtkWidget* focus_before_menu = NULL;
 static guint focus_notify_handler = 0;    
 
@@ -105,7 +110,7 @@ gail_get_accessible_for_widget (GtkWidget *widget,
   else if (GTK_IS_NOTEBOOK (widget)) 
     {
       GtkNotebook *notebook;
-      gint page_num;
+      gint page_num = -1;
 
       notebook = GTK_NOTEBOOK (widget);
       /*
@@ -179,6 +184,22 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
               window = GTK_WINDOW (widget);
               if (window->focus_widget)
                 {
+                  /*
+                   * If we already have a potential focus widget set this
+                   * windows's focus widget to focus_before_menu so that 
+                   * it will be reported when menu item is unset.
+                   */
+                  if (next_focus_widget)
+                    {
+                      if (GTK_IS_MENU_ITEM (next_focus_widget) &&
+                          !focus_before_menu)
+                        {
+                          focus_before_menu = window->focus_widget;
+                          g_object_add_weak_pointer (G_OBJECT (focus_before_menu), (gpointer *)&focus_before_menu);
+                        }
+
+                      return TRUE;
+                    }
                   widget = window->focus_widget;
                 }
               else if (window->type == GTK_WINDOW_POPUP)
@@ -249,27 +270,23 @@ gail_select_watcher (GSignalInvocationHint *ihint,
       g_signal_connect (widget, "map",
                         G_CALLBACK (gail_map_cb),
                         NULL);
-      return TRUE;
     }
+  else
+    gail_finish_select (widget);
 
-  gail_finish_select (widget);
   return TRUE;
 }
 
 static void
 gail_finish_select (GtkWidget *widget)
 {
-  if (focus_widget == widget)
-  /*
-   * We may already have got a notification because of bug 68483
-   */
-    return;
-
   /*
    * If previously focused widget is not a GtkMenuItem or a GtkMenu,
    * keep track of it so we can return to it after menubar is deactivated
    */
-  if (focus_widget && !GTK_IS_MENU_ITEM (focus_widget)  && !GTK_IS_MENU (focus_widget))
+  if (focus_widget && 
+      !GTK_IS_MENU_ITEM (focus_widget) && 
+      !GTK_IS_MENU (focus_widget))
     {
       focus_before_menu = focus_widget;
       g_object_add_weak_pointer (G_OBJECT (focus_before_menu), (gpointer *)&focus_before_menu);
@@ -301,7 +318,6 @@ gail_deselect_watcher (GSignalInvocationHint *ihint,
 
   widget = GTK_WIDGET (object);
 
-  
   if (!GTK_IS_MENU_ITEM (widget))
     return TRUE;
 
@@ -329,10 +345,51 @@ gail_deselect_watcher (GSignalInvocationHint *ihint,
   return TRUE; 
 }
 
+static gboolean 
+gail_switch_page_watcher (GSignalInvocationHint *ihint,
+                          guint                  n_param_values,
+                          const GValue          *param_values,
+                          gpointer               data)
+{
+  GObject *object;
+  GtkWidget *widget;
+  GtkNotebook *notebook;
+
+  object = g_value_get_object (param_values + 0);
+  g_return_val_if_fail (GTK_IS_WIDGET(object), FALSE);
+
+  widget = GTK_WIDGET (object);
+
+  if (!GTK_IS_NOTEBOOK (widget))
+    return TRUE;
+
+  notebook = GTK_NOTEBOOK (widget);
+  if (!notebook->focus_tab)
+    return TRUE;
+
+  gail_focus_notify_when_idle (widget);
+  return TRUE;
+}
+
+
 static gint
 gail_focus_idle_handler (gpointer data)
 {
   focus_notify_handler = 0;
+  /*
+   * The widget which was to receive focus may have been removed
+   */
+  if (!next_focus_widget)
+    {
+      if (next_focus_widget != data)
+        return FALSE;
+    }
+  else
+    {
+      g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
+      next_focus_widget = NULL;
+    }
+    
   gail_focus_notify (data);
 
   return FALSE; 
@@ -386,6 +443,8 @@ gail_focus_notify_when_idle (GtkWidget *widget)
       if (widget)
         {
           gtk_idle_remove (focus_notify_handler);
+          if (next_focus_widget)
+            g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
         }
       else
         /*
@@ -394,13 +453,19 @@ gail_focus_notify_when_idle (GtkWidget *widget)
         return;
     }
 
+  if (widget)
+    {
+      next_focus_widget = widget;
+      g_object_add_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
+    }
   focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, widget);
 }
+
 static gboolean
 gail_deactivate_watcher (GSignalInvocationHint *ihint,
-                    guint                  n_param_values,
-                    const GValue          *param_values,
-                    gpointer               data)
+                         guint                  n_param_values,
+                         const GValue          *param_values,
+                         gpointer               data)
 {
   GObject *object;
   GtkWidget *widget;
@@ -421,7 +486,6 @@ gail_deactivate_watcher (GSignalInvocationHint *ihint,
   return TRUE; 
 }
 
-
 static void
 gail_focus_tracker_init ()
 {
@@ -435,6 +499,7 @@ gail_focus_tracker_init ()
       gtk_type_class (GTK_TYPE_WIDGET);
       gtk_type_class (GTK_TYPE_ITEM);
       gtk_type_class (GTK_TYPE_MENU_SHELL);
+      gtk_type_class (GTK_TYPE_NOTEBOOK);
 
       /*
        * We listen for event_after signal and then check that the
@@ -468,6 +533,13 @@ gail_focus_tracker_init ()
              g_signal_lookup ("deactivate", GTK_TYPE_MENU_SHELL), 0,
              gail_deactivate_watcher, NULL, (GDestroyNotify) NULL);
 
+      /*
+       * We listen for "switch-page" signal on a GtkNotebook to notify
+       * when page has changed because of clicking on a notebook tab.
+       */
+      g_signal_add_emission_hook (
+             g_signal_lookup ("switch-page", GTK_TYPE_NOTEBOOK), 0,
+             gail_switch_page_watcher, NULL, (GDestroyNotify) NULL);
       emission_hooks_added = TRUE;
     }
 }
