@@ -84,7 +84,7 @@ gail_item_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-  accessible->role = ATK_ROLE_UNKNOWN;
+  accessible->role = ATK_ROLE_LIST_ITEM;
 
   return accessible;
 }
