@@ -20,6 +20,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include "gailmenuitem.h"
+#include "gailsubmenuitem.h"
 
 #define KEYBINDING_SEPARATOR ";"
 
@@ -115,6 +116,9 @@ static void
 gail_menu_item_real_initialize (AtkObject *obj,
                                 gpointer  data)
 {
+  GtkWidget *widget;
+  GtkWidget *parent;
+
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
   g_signal_connect (data,
@@ -125,6 +129,23 @@ gail_menu_item_real_initialize (AtkObject *obj,
                     "deselect",
                     G_CALLBACK (menu_item_deselect),
                     NULL);
+  widget = GTK_WIDGET (data);
+  parent = gtk_widget_get_parent (widget);
+  if (GTK_IS_MENU (parent))
+    {
+      GtkWidget *parent_widget;
+
+      parent_widget =  gtk_menu_get_attach_widget (GTK_MENU (parent));
+
+      if (!GTK_IS_MENU_ITEM (parent_widget))
+        parent_widget = gtk_widget_get_parent (widget);
+       if (parent_widget)
+        {
+          atk_object_set_parent (obj, gtk_widget_get_accessible (parent_widget));
+        }
+    }
+  g_object_set_data (G_OBJECT (obj), "atk-component-layer",
+                     GINT_TO_POINTER (ATK_LAYER_POPUP));
 }
 
 static void
@@ -142,6 +163,9 @@ gail_menu_item_new (GtkWidget *widget)
   
   g_return_val_if_fail (GTK_IS_MENU_ITEM (widget), NULL);
 
+  if (gtk_menu_item_get_submenu (GTK_MENU_ITEM (widget)))
+    return gail_sub_menu_item_new (widget);
+
   object = g_object_new (GAIL_TYPE_MENU_ITEM, NULL);
 
   accessible = ATK_OBJECT (object);
@@ -151,26 +175,21 @@ gail_menu_item_new (GtkWidget *widget)
     accessible->role = ATK_ROLE_TEAR_OFF_MENU_ITEM;
   else if (GTK_IS_SEPARATOR_MENU_ITEM (widget))
     accessible->role = ATK_ROLE_SEPARATOR;
-  if (gtk_menu_item_get_submenu (GTK_MENU_ITEM(widget)))
-    accessible->role = ATK_ROLE_MENU;
   else
     accessible->role = ATK_ROLE_MENU_ITEM;
 
-  g_object_set_data (G_OBJECT (accessible), "atk-component-layer",
-                     GINT_TO_POINTER (ATK_LAYER_POPUP));
-
   return accessible;
 }
 
-
 /*
- * If a menu item has a submenu return the submenu as the only
- * accessible child; otherwise expose no accessible children.
+ * If a menu item has a submenu return the items of the submenu as the 
+ * accessible children; otherwise expose no accessible children.
  */
 static gint
 gail_menu_item_get_n_children (AtkObject* obj)
 {
   GtkWidget *widget;
+  GtkWidget *submenu;
   gint count = 0;
 
   g_return_val_if_fail (GAIL_IS_MENU_ITEM (obj), count);
@@ -179,9 +198,15 @@ gail_menu_item_get_n_children (AtkObject* obj)
   if (widget == NULL)
     return count;
 
-  if (GTK_MENU_ITEM (widget)->submenu)
-    count = 1;
+  submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (widget));
+  if (submenu)
+    {
+      GList *children;
 
+      children = gtk_container_get_children (GTK_CONTAINER (submenu));
+      count = g_list_length (children);
+      g_list_free (children);
+    }
   return count;
 }
 
@@ -191,6 +216,7 @@ gail_menu_item_ref_child (AtkObject *obj,
 {
   AtkObject  *accessible;
   GtkWidget *widget;
+  GtkWidget *submenu;
 
   g_return_val_if_fail (GAIL_IS_MENU_ITEM (obj), NULL);
   g_return_val_if_fail ((i >= 0), NULL);
@@ -199,9 +225,21 @@ gail_menu_item_ref_child (AtkObject *obj,
   if (widget == NULL)
     return NULL;
 
-  if (GTK_MENU_ITEM (widget)->submenu && i == 0)
+  submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (widget));
+  if (submenu)
     {
-      accessible = gtk_widget_get_accessible (GTK_MENU_ITEM (widget)->submenu);
+      GList *children;
+      GList *tmp_list;
+
+      children = gtk_container_get_children (GTK_CONTAINER (submenu));
+      tmp_list = g_list_nth (children, i);
+      if (!tmp_list)
+        {
+          g_list_free (children);
+          return NULL;
+        }
+      accessible = gtk_widget_get_accessible (GTK_WIDGET (tmp_list->data));
+      g_list_free (children);
       g_object_ref (accessible);
     }
   else
@@ -215,6 +253,7 @@ gail_menu_item_ref_state_set (AtkObject *obj)
 {
   AtkStateSet *state_set;
   GtkWidget *widget;
+  GtkWidget *parent;
 
   state_set = ATK_OBJECT_CLASS (parent_class)->ref_state_set (obj);
   widget = GTK_ACCESSIBLE (obj)->widget;
@@ -225,6 +264,11 @@ gail_menu_item_ref_state_set (AtkObject *obj)
   if (!GTK_WIDGET_IS_SENSITIVE (widget))
     atk_state_set_remove_state (state_set, ATK_STATE_SELECTABLE);
 
+  parent = gtk_widget_get_parent (widget);
+  if (!GTK_WIDGET_VISIBLE (parent))
+    atk_state_set_remove_state (state_set, ATK_STATE_VISIBLE);
+  if (!GTK_WIDGET_MAPPED (parent))
+    atk_state_set_remove_state (state_set, ATK_STATE_SHOWING);
   return state_set;
 }
 
@@ -248,6 +292,7 @@ gail_menu_item_do_action (AtkAction *action,
   if (i == 0)
     {
       GtkWidget *item;
+      GtkWidget *item_parent;
 
       item = GTK_ACCESSIBLE (action)->widget;
       if (item == NULL)
@@ -257,11 +302,12 @@ gail_menu_item_do_action (AtkAction *action,
       if (!GTK_WIDGET_SENSITIVE (item) || !GTK_WIDGET_VISIBLE (item))
         return FALSE;
 
-      gtk_menu_shell_select_item (GTK_MENU_SHELL (item->parent), item);
+      item_parent = gtk_widget_get_parent (item);
+      gtk_menu_shell_select_item (GTK_MENU_SHELL (item_parent), item);
       /*
        * This is what is called when <Return> is pressed for a menu item
        */
-      g_signal_emit_by_name (item->parent, "activate_current",  
+      g_signal_emit_by_name (item_parent, "activate_current",  
                                /*force_hide*/ 1); 
       return TRUE;
     }
