@@ -46,6 +46,7 @@ static AtkObject*   find_child_in_list                (GList          *list,
 static gboolean     gail_notebook_focus_cb            (GtkWidget      *widget,
                                                        GtkDirectionType type);
 static gboolean     gail_notebook_check_focus_tab     (gpointer       data);
+static void         gail_notebook_destroyed           (gpointer       data);
 
 static gpointer parent_class = NULL;
 
@@ -115,6 +116,7 @@ gail_notebook_object_init (GailNotebook      *notebook)
   notebook->page_cache = NULL;
   notebook->selected_page = -1;
   notebook->focus_tab_page = -1;
+  notebook->idle_focus_id = 0;
 }
 
 AtkObject* 
@@ -212,6 +214,9 @@ gail_notebook_real_initialize (AtkObject *obj,
                     "focus",
                     G_CALLBACK (gail_notebook_focus_cb),
                     NULL);
+  g_object_weak_ref (G_OBJECT(gtk_notebook),
+                     (GWeakNotify) gail_notebook_destroyed,
+                     obj);                     
 }
 
 static void
@@ -264,14 +269,12 @@ gail_notebook_real_notify_gtk (GObject           *obj,
                */
             }
         }
-      if (focus_page_num != old_focus_page_num)
+      if (gtk_notebook_get_show_tabs (gtk_notebook) &&
+         (focus_page_num != old_focus_page_num))
         {
-          AtkObject *obj;
-
-          if (gtk_notebook_get_show_tabs (gtk_notebook)) {
-            obj = find_child_in_list (gail_notebook->page_cache, focus_page_num);
-            atk_focus_tracker_notify (obj);
-          }
+          if (gail_notebook->idle_focus_id)
+            gtk_idle_remove (gail_notebook->idle_focus_id);
+          gail_notebook->idle_focus_id = gtk_idle_add (gail_notebook_check_focus_tab, atk_obj);
         }
     }
   else
@@ -442,11 +445,16 @@ static gboolean
 gail_notebook_focus_cb (GtkWidget      *widget,
                         GtkDirectionType type)
 {
+  AtkObject *atk_obj = gtk_widget_get_accessible (widget);
+  GailNotebook *gail_notebook = GAIL_NOTEBOOK (atk_obj);
+
   switch (type)
     {
     case GTK_DIR_LEFT:
     case GTK_DIR_RIGHT:
-      gtk_idle_add (gail_notebook_check_focus_tab, widget);
+      if (gail_notebook->idle_focus_id)
+        gtk_idle_remove (gail_notebook->idle_focus_id);
+      gail_notebook->idle_focus_id = gtk_idle_add (gail_notebook_check_focus_tab, atk_obj);
       break;
     default:
       break;
@@ -457,11 +465,22 @@ gail_notebook_focus_cb (GtkWidget      *widget,
 static gboolean
 gail_notebook_check_focus_tab (gpointer data)
 {
-  GtkWidget *widget = GTK_WIDGET (data);
-  AtkObject* atk_obj = gtk_widget_get_accessible (widget);
+  GtkWidget *widget;
+  AtkObject *atk_obj;
   gint focus_page_num, old_focus_page_num;
-  GailNotebook *gail_notebook = GAIL_NOTEBOOK (atk_obj);
-  GtkNotebook *gtk_notebook = GTK_NOTEBOOK (widget);
+  GailNotebook *gail_notebook;
+  GtkNotebook *gtk_notebook;
+
+  atk_obj = ATK_OBJECT (data);
+  gail_notebook = GAIL_NOTEBOOK (atk_obj);
+  widget = GTK_ACCESSIBLE (atk_obj)->widget;
+
+  gtk_notebook = GTK_NOTEBOOK (widget);
+
+  gail_notebook->idle_focus_id = 0;
+
+  if (!gtk_notebook->focus_tab)
+    return FALSE;
 
   old_focus_page_num = gail_notebook->focus_tab_page;
   focus_page_num = g_list_index (gtk_notebook->children, gtk_notebook->focus_tab->data);
@@ -470,9 +489,18 @@ gail_notebook_check_focus_tab (gpointer data)
     {
       AtkObject *obj;
 
-      obj = find_child_in_list (gail_notebook->page_cache, focus_page_num);
+      obj = atk_object_ref_accessible_child (atk_obj, focus_page_num);
       atk_focus_tracker_notify (obj);
+      g_object_unref (obj);
     }
   return FALSE;
 }
 
+static void
+gail_notebook_destroyed (gpointer data)
+{
+  GailNotebook *gail_notebook = GAIL_NOTEBOOK (data);
+
+  if (gail_notebook->idle_focus_id)
+    gtk_idle_remove (gail_notebook->idle_focus_id);
+}
