@@ -79,11 +79,6 @@ gail_option_menu_get_type (void)
 static void
 gail_option_menu_class_init (GailOptionMenuClass *klass)
 {
-  AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
-  GailWidgetClass *widget_class;
-
-  widget_class = (GailWidgetClass*)klass;
-
   parent_class = g_type_class_peek_parent (klass);
 }
 
@@ -132,6 +127,9 @@ gail_option_menu_do_action (AtkAction *action,
      */
     return FALSE;
 
+  if (!GTK_WIDGET_SENSITIVE (widget) || !GTK_WIDGET_VISIBLE (widget))
+    return FALSE;
+
   button = GTK_BUTTON (widget); 
   switch (i)
     {
