@@ -269,6 +269,8 @@ gail_text_cell_update_cache (GailRendererCell *cell,
               if (obj->name == NULL)
                 g_object_notify (G_OBJECT (obj), "accessible-name");
             }
+          if (new_cache)
+            rv = TRUE;
         }
     }
   else
@@ -610,8 +612,10 @@ gail_text_cell_get_character_extents (AtkText          *text,
   pango_layout_index_to_pos (layout, index, &char_rect); 
 
   gail_misc_get_extents_from_pango_rectangle (widget,
-      &char_rect, x_offset + rendered_rect.x,
-      y_offset + rendered_rect.y, x, y, width, height, coords);
+      &char_rect,
+      x_offset + rendered_rect.x + gail_renderer->renderer->xpad,
+      y_offset + rendered_rect.y + gail_renderer->renderer->ypad,
+      x, y, width, height, coords);
   g_object_unref (layout);
   return;
 } 
@@ -628,7 +632,7 @@ gail_text_cell_get_offset_at_point (AtkText          *text,
   GtkWidget *widget;
   GdkRectangle rendered_rect;
   PangoLayout *layout;
-  gint x_offset, y_offset, index, cell_height, cell_width;
+  gint x_offset, y_offset, index;
  
   if (!GAIL_TEXT_CELL (text)->cell_text)
     return -1;
@@ -647,16 +651,22 @@ gail_text_cell_get_offset_at_point (AtkText          *text,
   gail_cell_parent_get_cell_area (GAIL_CELL_PARENT (parent), GAIL_CELL (text),
                                   &rendered_rect);
   gtk_cell_renderer_get_size (GTK_CELL_RENDERER (gtk_renderer), widget,
-     &rendered_rect, &x_offset, &y_offset, &cell_width, &cell_height);
+     &rendered_rect, &x_offset, &y_offset, NULL, NULL);
 
   layout = create_pango_layout (gtk_renderer, widget);
    
   index = gail_misc_get_index_at_point_in_layout (widget, layout,
-        x_offset + rendered_rect.x, y_offset + rendered_rect.y,
+        x_offset + rendered_rect.x + gail_renderer->renderer->xpad,
+        y_offset + rendered_rect.y + gail_renderer->renderer->ypad,
         x, y, coords);
   g_object_unref (layout);
   if (index == -1)
-    return index;  
+    {
+      if (coords == ATK_XY_WINDOW || coords == ATK_XY_SCREEN)
+        return g_utf8_strlen (gtk_renderer->text, -1);
+    
+      return index;  
+    }
   else
     return g_utf8_pointer_to_offset (gtk_renderer->text,
        gtk_renderer->text + index);  
@@ -667,11 +677,17 @@ gail_text_cell_get_character_at_offset (AtkText       *text,
                                         gint          offset)
 {
   gchar *index;
+  gchar *string;
 
-  if (GAIL_TEXT_CELL(text)->cell_text == NULL)
+  string = GAIL_TEXT_CELL(text)->cell_text;
+
+  if (!string)
     return '\0';
 
-  index = g_utf8_offset_to_pointer (GAIL_TEXT_CELL (text)->cell_text, offset);
+  if (offset >= g_utf8_strlen (string, -1))
+    return '\0';
+
+  index = g_utf8_offset_to_pointer (string, offset);
 
   return g_utf8_get_char (index);
 }
