@@ -18,6 +18,7 @@
  */
 
 #include <stdlib.h>
+#include <string.h>
 #include <gtk/gtk.h>
 #include "gailtoplevel.h"
 
@@ -248,7 +249,8 @@ gail_toplevel_show_event_watcher (GSignalInvocationHint *ihint,
   * but g_list_length is 1-based.
   */
   g_signal_emit_by_name (atk_obj, "children-changed::add",
-                         n_children - 1, NULL, NULL);
+                         n_children - 1, 
+                         gtk_widget_get_accessible (widget), NULL);
 
   /* Connect destroy signal callback */
   g_signal_connect (G_OBJECT(object), 
@@ -304,7 +306,8 @@ _gail_toplevel_remove_child (GailToplevel *toplevel,
             {
               /* Remove the window from the window_list & emit the signal */
               g_signal_emit_by_name (atk_obj, "children-changed::remove",
-                                     window_count, NULL, NULL);
+                                     window_count, 
+                                     gtk_widget_get_accessible (GTK_WIDGET (window)), NULL);
               toplevel->window_list = g_list_remove (toplevel->window_list,
                                                      l->data);
               break;
