@@ -105,7 +105,7 @@ gint setup_gui(AtkObject *obj, TLruntest test)
   add_test(window, "atk_text_get_offset_at_point", 3, paramnames, defaults);
 
   /* Editable Text Tests */
-  if (ATK_IS_EDITABLE_TEXT(obj) == TRUE) 
+  if (ATK_IS_EDITABLE_TEXT(obj)) 
   {
 
     paramnames[0] = "start";
@@ -593,7 +593,7 @@ runtest(AtkObject *obj, gint win_val)
            
       set_output_buffer(output);
     } 
-    if (ATK_IS_EDITABLE_TEXT(obj) == TRUE)
+    if (ATK_IS_EDITABLE_TEXT(obj))
     {
       if (strcmp(testsOn[i], "atk_editable_text_set_run_attributes") == 0)
       {
